#ifndef ASSERT
#define ASSERT
#include <assert.h>
#endif

#ifndef COMMONS
#define COMMONS
#include "commons.c"
#endif

#ifndef T_LISTS
#define T_LISTS
#include "t_lists.c"
#endif

#ifndef ARRAYLIST
#define ARRAYLIST
#include "arraylist.c"
#endif

/*--------------------------------------------------------------------------
 * Auxiliary Functions
 *--------------------------------------------------------------------------
 */
void collections_swap(PArrayList list,
		int i,
		int j)
{
	PObject val_i = arraylist_getIndex(list, i);
	PObject val_j = arraylist_setIndex(list, j, val_i);
	arraylist_setIndex(list, i, val_j);
}

/*--------------------------------------------------------------------------
 * replaceAll()
 *--------------------------------------------------------------------------
 */
void collections_replaceAll(PArrayList list,
	PObject oldVal,
	PObject newVal)
{
	if (oldVal==newVal)
		return;

//	assert(list!=NULL);	//SAFE:1.319s
	int size = list->size;
	int ic = 0;
	for ( ; ic < size; ic++) {
//		assert(list!=NULL); //SAFE:1.319s
//		assert (ic>=0 && ic<list->size);	//SAFE:0.398s
		PObject val = arraylist_getIndex(list, ic);
		if (val==oldVal) {
//			assert(list!=NULL);//SAFE:1.319s
//			assert(ic>=0 && ic<list->size);	//SAFE:0.522s
			arraylist_setIndex(list, ic, newVal);
		}
	}
}

void blast_replaceAll(PArrayList list,
	PObject oldVal,
	PObject newVal)
{
	if (list==NULL)
		return;

	if (!arraylist_invariants(list))
		return;

	PObject* old_elementData = arrays_clone(list->elementData, list->elementData_length);
	collections_replaceAll(list, oldVal, newVal);

//	assert ( oldVal!=newVal || t_lists_eq2(list->elementData, old_elementData, 0, list->size));	//UNSAFE:1.121s
//	assert ( oldVal==newVal || t_lists_neq(list->elementData, 0, list->size, oldVal));	//UNSAFE:0.532s

}

/*--------------------------------------------------------------------------
 * sort(ArrayList)
 *--------------------------------------------------------------------------
 */
void collections_sort(PArrayList list)
{
	int a_length;
//	assert(list!=NULL);	//SAFE:1.389s
	PObject* a =arraylist_toArray(list, &a_length);
//	assert(a==NULL);	//CRASH:3s
	arrays_sort(a, a_length);
	
	arraylist_clear(list);
	int ic = 0;
	for ( ; ic < a_length; ic++) {
//		assert (a!=NULL);	//CRASH:5.193s
//		assert (ic>=0);	//SAFE:0.351s
//		assert (ic< a_length);	//CRASH:5.101s
		PObject elem = a[ic];

//		assert(list!=NULL);	//SAFE:0.317s
		arraylist_add(list, elem);
	}
}

void blast_sort(PArrayList list)
{
	if (list==NULL)
		return;

	if (!arraylist_invariants(list))
		return;

	if (!t_lists_neq(list->elementData, 0, list->size, NULL))
		return;

	int old_size = list->size;
	collections_sort(list);

//	assert(old_size==list->size);	//CRASH:4.903s
//	assert(t_lists_neq(list->elementData, 0, list->size, NULL));	//CRASH:5.135s
//	assert(t_lists_sorted(list->elementData, list->elementData_length, 0, list->size));	//CRASH:4.98s
}
/*--------------------------------------------------------------------------
 * reverse(ArrayList)
 *--------------------------------------------------------------------------
 */
void collections_reverse(PArrayList list)
{
//	assert (list!=NULL);	//CRASH:1.922s
	int size = list->size;
	int mid = size /2;
	int ic = 0;
	for ( ; ic <mid; ic++) {
//		assert (list!=NULL);	//CRASH:1.619s
//		assert (0<=ic && ic<list->size);	//CRASH:0.356s
//		assert (0<=(size-1-ic) && (size-1-ic)<list-size);	//CRASH:0.350s
		collections_swap(list, ic, size-1-ic);
	}
}

void blast_reverse(PArrayList list)
{
	if (list==NULL)
		return;

	if (!arraylist_invariants(list))
		return;

	PObject* old_elementData = arrays_clone(list->elementData, list->elementData_length);
	collections_reverse(list);

	assert(t_lists_is_reverse(list->elementData, old_elementData, 0, list->size));	//CRASH:1.063s
}
