#ifndef ASSERT
#define ASSERT
#include <assert.h>
#endif

#ifndef COMMONS
#define COMMONS
#include "commons.c"
#endif

#ifndef T_LISTS
#define T_LISTS
#include "t_lists.c"
#endif

#ifndef ARRAYS
#define ARRAYS
#include "arrays.c"
#endif

typedef struct Struct_Vector {
        int modCount;
        int elementCount;
	int capacityIncrement;
        PObject* elementData;
        int elementData_length;
} Vector;

typedef Vector* PVector;

/*------------------------------------------------------
 * Contracts
 *------------------------------------------------------
 */
//@ invariant elementData != null;
bool vector_invariant_1(PVector vector) 
{       
        return (vector->elementData!=NULL);
}

//@ invariant (\forall int i; elementCount <= i && i < elementData.length ==> elementData[i] == null);
bool vector_invariant_2(PVector vector)
{
	int i = vector->elementCount;
	for ( ; i < vector->elementData_length; i++) {
		if (vector->elementData[i]!=NULL) {
			return FALSE;
		}
	}
	return TRUE;
}

//@ invariant 0 <= elementCount && elementCount <= elementData.length;
bool vector_invariant_3(PVector vector)
{
	return 0<=vector->elementCount && vector->elementCount<=vector->elementData_length;
}
//@ invariant capacityIncrement >= 0;
bool vector_invariant_4(PVector vector) 
{
	return vector->capacityIncrement>=0;
}

bool vector_invariants(PVector vector)
{
	return vector_invariant_1(vector) 
		&& vector_invariant_2(vector)
		&& vector_invariant_3(vector)
		&& vector_invariant_4(vector);
}

/*------------------------------------------------------------------------
 * indexOf1(Integer value, int index);
 *------------------------------------------------------------------------
 */
int vector_indexOf1(PVector vector,
	PObject value,
	int index,
	PErrorCode error_code)
{
	if (index<0 || index > vector->elementCount) {
		(*error_code) = ERROR;
		return -1;
	}

	if (value==NULL) {
		int ic = index;
		for ( ; ic < vector->elementCount; ic++) {
//			assert (vector->elementData!=NULL);	//SAFE:0.429s
//			assert (ic>=0);	//SAFE:0.758s
//			assert (ic<vector->elementData_length);	//SAFE:0.758s
			if ( vector->elementData[ic]==NULL) {
				return ic;
			}
		}
	} else {
		int ic = index;
		for ( ; ic < vector->elementCount; ic++) {
//			assert (vector->elementData!=NULL);	//SAFE:0.443s
//			assert (ic>=0);	//SAFE:0.758s
//			assert (ic<vector->elementData_length);	//SAFE:0.758s
			if ( vector->elementData[ic]==value) {
				return ic;
			}
		}
	}
	return -1;
}


void blast_indexOf1(PVector vector,
	PObject value,
	int index)
{
	ErrorCode error_code = OK;

	if (vector==NULL)
		return;

	if (!vector_invariants(vector))
		return;

	if (index<0 || index>vector->elementCount)
		return;

	int result = vector_indexOf1(vector, value, index, &error_code);

	if (error_code==OK) {
	//	assert ((result==-1) == (!t_lists_in(vector->elementData,index,vector->elementCount,value))); 	//UNSAFE:3.693s
	//	assert (result==-1 || (index<=result && result <vector->elementCount));	//SAFE:0.657s
	//	assert (result==-1 || (vector->elementData[result]==value && !t_lists_in(vector->elementData, index, result, value)));	//UNSAFE:0.408s
	}
//	assert(vector_invariant_1(vector));	//SAFE:0.647s
//	assert(vector_invariant_2(vector));	//UNSAFE:1.542s
//	assert(vector_invariant_3(vector));	//SAFE:0.942s
//  assert(vector_invariant_4(vector));	//SAFE:0.683s
	
} 

/*------------------------------------------------------------------------
 * lastIndexOf1(Integer value, int index);
 *------------------------------------------------------------------------
 */
int vector_lastIndexOf1(PVector vector,
	PObject value,
	int index,
	PErrorCode error_code)
{
	if (index<0 || index > vector->elementCount) {
		(*error_code) = ERROR;
		return -1;
	}

	if (value==NULL) {
		int ic = index;
		for ( ; ic >=0; ic--) {
//			assert (vector->elementData!=NULL);	//SAFE:0.444s
//			assert (ic>=0);	//SAFE:0.636s
//			assert (ic<vector->elementData_length);	//SAFE:0.636s
			if ( vector->elementData[ic]==NULL) {
				return ic;
			}
		}
	} else {
		int ic = index;
		for ( ; ic >= 0; ic--) {
//			assert (vector->elementData!=NULL);	//SAFE:0.452s
//			assert (ic>=0);	//SAFE:0.612s
//			assert (ic<vector->elementData_length);	//SAFE:0.612s
			if ( vector->elementData[ic]==value) {
				return ic;
			}
		}
	}
	return -1;
}


void blast_lastIndexOf1(PVector vector,
	PObject value,
	int index)
{
	ErrorCode error_code = OK;
	if (vector==NULL)
		return;

	if (!vector_invariants(vector))
		return;

	if (index<0 || index>=vector->elementCount)
		return;

	int result = vector_lastIndexOf1(vector, value, index, &error_code);

	if (error_code==OK) {
//		assert ((result==-1) == (!t_lists_in(vector->elementData,0,index+1,value))); 	//UNSAFE:1.859s
//		assert (result==-1 || (0<=result && result <= index));	//SAFE:0.442s
//		assert (result==-1 || (vector->elementData[result]==value && !t_lists_in(vector->elementData, result+1, index+1, value)));	//UNSAFE:0.389s
	}
	
//	assert(vector_invariant_1(vector));	//SAFE:0.783s
//	assert(vector_invariant_2(vector));	//UNSAFE:1.823s
//	assert(vector_invariant_3(vector));	//SAFE:1.144s
//  assert(vector_invariant_4(vector));	//SAFE:0.924s

}

/*------------------------------------------------------------------------
 * removeAllElements()
 *------------------------------------------------------------------------
 */
void vector_removeAllElements(PVector vector)
{
	vector->modCount++;
	int ic = 0;
	for ( ; ic<vector->elementCount; ic++) {
//		assert (vector->elementData!=NULL);	//SAFE:0.388s
//		assert (ic>=0);	//SAFE:0.648s
//		assert (ic<vector->elementData_length);	//SAFE:0.648s
		vector->elementData[ic] = NULL;
	}
}

void blast_removeAllElements(PVector vector)
{
	if (vector==NULL)
		return;

	if (!vector_invariants(vector))
		return;

	int old_elementCount = vector->elementCount;
	vector_removeAllElements(vector);

//	assert (vector->elementCount==0);	//CRASH:3.115s
//	assert (t_lists_eq(vector->elementData, 0, old_elementCount, NULL));	//UNSAFE:0.510s
//	assert (t_lists_eq(vector->elementData, old_elementCount, vector->elementData_length, NULL));	//UNSAFE:1.029s

//	assert(vector_invariant_1(vector));	//SAFE:0.482s
//	assert(vector_invariant_2(vector));	//UNSAFE:1.104s
//	assert(vector_invariant_3(vector));	//SAFE:0.740s
//        assert(vector_invariant_4(vector));	//SAFE:0.546s

}

/*------------------------------------------------------------------------
 * removeRange0(int fromIndex, int toIndex)
 *------------------------------------------------------------------------
 */
void vector_removeRange0(PVector vector, 
	int fromIndex,
	int toIndex)
{
	vector->modCount++;
	int numMoved = vector->elementCount - toIndex;

//	assert(vector->elementData != NULL && vector->elementData != NULL);	//SAFE:3.043s
//	assert(numMoved >= 0);	//SAFE:0.741s
//	assert(toIndex >= 0 && toIndex + numMoved <= vector->elementData_length);	//SAFE:2.46s
//	assert(fromIndex >= 0 && fromIndex + numMoved <= vector->elementData_length);	//SAFE:2.383s

	system_arraycopy(vector->elementData,
		vector->elementData_length,
		toIndex,
		vector->elementData,
		vector->elementData_length,
		fromIndex,
		numMoved);

	int oldElementCount = vector->elementCount;
	int newElementCount = oldElementCount - (toIndex - fromIndex);
	
	while (vector->elementCount != newElementCount) {
		vector->elementCount--;
//		assert (vector->elementData!=NULL);	//SAFE:3.466s
//		assert (vector->elementCount>=0);	//CRASH:12.763s
//		assert (vector->elementCount<vector->elementData_length);	//SAFE:2.763s
		vector->elementData[vector->elementCount]=NULL;
	}
}

bool ensures_removeRange0(PObject* elementData,
	PObject* old_elementData,
	int old_elementCount,
	int fromIndex,
	int toIndex)
{
	int i =0;
	for ( ; i < (old_elementCount - toIndex); i++) {
		if (elementData[fromIndex+i]!=old_elementData[toIndex+i]) {
			return FALSE;
		}
	}
	return TRUE;
} 

void blast_removeRange0(PVector vector,
	int fromIndex,
	int toIndex)
{
	if (vector==NULL)
		return;

	if (!vector_invariants(vector))
		return;

	if (!( 0 <= fromIndex && fromIndex <= toIndex && toIndex < vector->elementCount))
		return;

	int old_elementCount = vector->elementCount;
	PObject* old_elementData = arrays_clone(vector->elementData, 
		vector->elementData_length);

	vector_removeRange0(vector,fromIndex,toIndex);

//	assert (vector->elementCount==old_elementCount - (toIndex - fromIndex));	//UNSAFE:17.443s
//	assert (t_lists_eq2(vector->elementData, old_elementData, 0, fromIndex));	//UNSAFE:7.705s
//	assert (ensures_removeRange0(vector->elementData, old_elementData, old_elementCount, fromIndex, toIndex));	//UNSAFE:13.189s
//	assert (t_lists_eq(vector->elementData, vector->elementCount, vector->elementData_length, NULL));	//UNSAFE:25.667s
//
//	assert(vector_invariant_1(vector));	//SAFE:2.390s
//	assert(vector_invariant_2(vector));	//UNSAFE:30.344s
//	assert(vector_invariant_3(vector));	//SAFE:1.341s
//        assert(vector_invariant_4(vector));	//SAFE:0.915s

}
/*------------------------------------------------------------------------
 * setSize(int newSize)
 *------------------------------------------------------------------------
 */
void vector_ensureCapacityHelper(PVector vector,
	int minCapacity)
{
	int oldCapacity = vector->elementData_length;
	if (minCapacity > oldCapacity) {
		PObject* oldData = vector->elementData;
		int newCapacity;
		if (vector->capacityIncrement>0) {
			newCapacity = oldCapacity + vector->capacityIncrement;
		} else {
			newCapacity = oldCapacity *2;
		}
		if (newCapacity < minCapacity) {
			newCapacity = minCapacity;
		}
		vector->elementData = arrays_copyOf(vector->elementData, 
					vector->elementData_length,
					newCapacity);

		vector->elementData_length = newCapacity;
	}
}

void vector_setSize(PVector vector,
	int newSize,
	PErrorCode error_code)
{
	if (newSize<0) {
		(*error_code) = ERROR;
		return;
	}

	vector->modCount++;
	if (newSize > vector->elementCount) {
		vector_ensureCapacityHelper(vector, newSize);
	} else {
		int ic = newSize;
		for ( ; ic < vector->elementCount; ic++) {
//			assert (vector->elementData!=NULL);	//SAFE:2.174s
//			assert (ic>=0);	//SAFE:1.101s
//			assert (ic<vector->elementData_length);	//SAFE:1.101s
			vector->elementData[ic]=NULL;
		}
	}
	vector->elementCount = newSize;
}


void blast_setSize(PVector vector,
	int newSize)
{
	ErrorCode error_code = OK;
	if (vector==NULL)
		return;

	if (!vector_invariants(vector))
		return;

	if (newSize<0)
		return;

	int old_elementCount = vector->elementCount;
	PObject* old_elementData = arrays_clone(vector->elementData, 
		vector->elementData_length);

	vector_setSize(vector,newSize, &error_code);

	if (error_code==OK) {
//		assert (vector->elementCount==newSize);	//UNSAFE:3.013s
//		assert ( (newSize > old_elementCount || t_lists_eq2(vector->elementData, old_elementData, 0, newSize)) && (newSize <=old_elementCount || t_lists_eq2(vector->elementData, old_elementData, 0, old_elementCount)) ); // UNSAFE:2.138s
//		assert (newSize >=old_elementCount || t_lists_eq(vector->elementData, newSize, vector->elementData_length, NULL));	//UNSAFE:2.135s
	}

//	assert(vector_invariant_1(vector));	//SAFE:0.886s
//	assert(vector_invariant_2(vector));	//UNSAFE:4.912s
//	assert(vector_invariant_3(vector));	//SAFE:1.782s
//    assert(vector_invariant_4(vector));	//SAFE:0.951s

}
